BusMap.Predictions = function(opts) {
    var stops = {};
    var routes = {};
    var that = this.Predictions;
    that.opts = opts;
    window._buspredictions = this.Predictions;

    /* Constructor - create/initialize the map */
    function init() {
        // Fetch initial data
        updateRoutes();
        updatePredictions();

        // Begin timed data updates
        if (that.opts.refresh.routes) {
            setInterval(updateRoutes, that.opts.refresh.routes * 1000);
        }
        if (that.opts.refresh.predictions) {
            setInterval(updatePredictions, that.opts.refresh.predictions * 1000);
        }

        // Listen for hash change
        if (that.opts.hashListen !== false) {
            $(window).on('hashchange', setViewFromUrl);
        }

        // Grab initial URL
        setViewFromUrl();

    };

    /* Get Routes (and Stops, and Directions) */
    function updateRoutes() {
        var url = "ajax";
        var params = {
            query: "routes",
            agency: that.opts.agency,
        };
        $.getJSON(url, params)
            .done(function(data) {
                that.stops = data.stops;
                that.routes = data.routes;
                updateStopsUI(that.stops);
            });
    };

    /* Get Vehicles (and Predictions) */
    function updatePredictions() {
        if (!that.stops) {
            updateRoutes();
            return setTimeout(function() { updatePredictions() }, 500);
        }
        var url = "ajax";
        var params = {
            query: "predictions",
            stops: that.stop,
            agency: that.opts.agency,
        };
        $.getJSON(url, params)
            .done(function(data) {
                // Store predictions
                for (var s in that.stops) {
                    that.stops[s].predictions = {};
                }
                for (var p in data.predictions) {
                    var pr = data.predictions[p];
                    if (that.stops && pr.stop_id in that.stops) {
                        // Store this prediction with the relevant stop
                        if (!(pr.route in that.stops[pr.stop_id].predictions)) {
                            that.stops[pr.stop_id].predictions[pr.route] = [];
                        }
                        that.stops[pr.stop_id].predictions[pr.route].push(pr);
                    }
                }
                updatePredictionsUI(that.stops);
            });
    };

    function applyViewString(view) {
        if (!view || view == "") return false;
        if (!that.stops) {
            updateRoutes();
            return setTimeout(function() { applyViewString(view) }, 500);
        }
        if (view.charAt(0).toLowerCase() == "s") return _avsStop(view);
        function _avsStop(view) {
            var stop = that.stops[view.substring(1)];
            if (stop) {
                that.stop = stop.id;
                // Show this stop
                $(that.opts.predictionsElement).html(
                    "<header>" + stop.title + "</header>"
                  + "<p></p>");
                updatePredictions();
            } else console.log('Unknown stop in view string: ' + view.substring(1));
        }
    }

    function setViewFromUrl() {
        var hash = window.location.hash.substring(1);
        if (hash == "") {
            $(that.opts.predictionsElement).html("No stop selected");
        } else applyViewString(hash);
    }

    /* Refresh (and/or create) UI elements for Stops */
    function updateStopsUI(stops) {
        var hash = window.location.hash.substring(1);
        if (hash == "") return false;
        applyViewString(hash);
    }

    /* Refresh (and/or create) UI elements for Predictions */
    function updatePredictionsUI(stops) {
        if (!that.stop) return false;
        var s = that.stop;
        if (stops[s].predictions) {
            var predictions = [];
            var now = new Date();
            var offset_mins = now.getTimezoneOffset();
            for (var r in stops[s].predictions) {
                if (!(r in that.routes)) {
                    console.log("Unknown route " + r + " for stop " + stops[s].title);
                }
                var p_line = "<strong>" + that.routes[r].title + "</strong>: ";
                var times = [];
                // Sort by estimated time to arrival
                var psorted = stops[s].predictions[r].sort(function(a,b){
                    return new Date(a.prediction).getTime() - new Date(b.prediction).getTime();
                });
                for (var p in psorted) {
                    var pr = psorted[p];
                    var pdate = new Date(pr.prediction);
                    var diff_sec = (pdate.getTime() - now.getTime()) / 1000;
                    var diff_min = Math.ceil(diff_sec / 60) + offset_mins;
                    // CSS classes for predictions
                    var pclass = "";
                    if (diff_min <= 1) { pclass = "lt1min"; }
                    else if (diff_min <= 2 ) { pclass = "lt2mins"; }
                    else if (diff_min <= 5 ) { pclass = "lt5mins"; }
                    times.push("<span class='prediction " + pclass
                             + "' title='" + pr.vehicle + "'>"
                             + diff_min + "</span>");
                }
                p_line += times.join(", ");
                predictions.push("<span class='route'" + p_line + "</span>");
            }
            if (predictions.length == 0) {
                var predictions = ['<span class="none">No arrival predictions.</span>'];
            }
            var text = '<section class="predictions stop-predictions">'
                  + predictions.sort().join("<br>") + '</section>';
            $(that.opts.predictionsElement).find('p').html(text);
        }
    }

    init();
    return that;
}
